﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Forms;

namespace FormClient
{
    public partial class FrmClient : Form
    {
        public FrmClient()
        {
            InitializeComponent();
        }
        /// <summary>
        /// Pseudo
        /// </summary>
        string pseudo;
        /// <summary>
        /// Socket du client
        /// </summary>
        Socket clientSocket;
        /// <summary>
        /// Endpoint du serveur
        /// </summary>
        EndPoint serverEP;

        /// <summary>
        /// Function exectuée dans le thread : On ecoute le réseau et traite les messages reçu
        /// </summary>
        /// <param name="client">Socket du client</param>
        /// <param name="server">EndPoint du serveur</param>
        private void readNet(Socket client, EndPoint server)
        {
            try
            {

                byte[] buffer = new byte[1500];
                while (true)
                {
                    int nBytes = client.ReceiveFrom(buffer, buffer.Length, SocketFlags.None, ref server);
                    // Message reçu
                    ChatMessage receive = new ChatMessage(buffer);
                    Action action;

                    // On vide la rtb si on recoit la reponse d'un GET
                    if (receive.commandeType == CommandeType.REPONSE && receive.commande == Commande.GET){
                        action = () => rtbMSG.Text = "";
                        // Permet d'effacer tout les messages affichés dans le form
                        this.Invoke(action);
                    }

                    if (receive.commandeType == CommandeType.REPONSE){
                        // On affiche dans la console
                        Console.WriteLine(" < Reponse du serveur > " + receive.data);
                    }
                    else
                    {
                        // Split du message POST afin d'obtenir le pseudo et le message séparement
                        string[] splitted = receive.data.Split('\0');
                       
                        if (receive.commande == Commande.GET)
                        {
                            // GET
                            action = () => rtbMSG.Text = '\n' + " <" + splitted[0] + "> " + splitted[1];
                            this.Invoke(action);
                        }
                        else
                        {
                            // AUTRES
                            action = () => rtbMSG.Text += '\n' + " <" + splitted[0] + "> " + splitted[1];
                            this.Invoke(action);
                            Console.WriteLine(" <" + splitted[0] + "> " + splitted[1]);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                // SERVEUR OFF => On ferme le client
                Console.WriteLine("ERROR READ NET : Serveur fermé");
                MessageBox.Show("Le serveur n'est pas disponible! Fermeture du client...");
                Application.Exit();
            }

        }

        /// <summary>
        ///  Se declenche au demarrage : initialisation du socket et des info utilisateurs
        /// </summary>
        private void Form1_Load(object sender, EventArgs e)
        {
            // ======================================== INITIALISATION ============================
            // On demande le port du client et son pseudo
            Console.WriteLine("Port(pas 11111)?");
            int portClient = Convert.ToInt32(Console.ReadLine());
            Console.WriteLine('\n' + "Pseudo?");
            pseudo = Console.ReadLine();

            // Si pseudo trop long => On truncate
            if (pseudo.Length > 30)
                pseudo = pseudo.Substring(0, 30);
            // ===================================================================================

            Console.WriteLine("=============================================");
            Console.WriteLine("=============================================");
            Console.WriteLine("================== CLIENT ===================");
            Console.WriteLine("=============================================");
            Console.WriteLine("=============================================" + '\n' + '\n');

            // Coordonnées du serveur
            string serverIP = "127.0.0.1";
            int serverPort = 11111;

            // Création de la socket du client
            clientSocket = new Socket(
                AddressFamily.InterNetwork,
                SocketType.Dgram,
                ProtocolType.Udp);

            // Liaison de la socket au point de communication
            clientSocket.Bind(new IPEndPoint(IPAddress.Any, portClient));

            // Création du EndPoint serveur
            serverEP = new IPEndPoint(IPAddress.Parse(serverIP), serverPort);

            // Creation et lancement du thread d'ecoute
            Thread sendAll = new Thread(x => readNet(clientSocket, serverEP));
            sendAll.Start();

            // On change le focus
            this.ActiveControl = txtMsg;
        }
        
        /// <summary>
        /// Se declenche au click du bouton envoi: Envoi du message POST
        /// </summary>
        private void btnPOST_Click(object sender, EventArgs e)
        {
            ChatMessage m = new ChatMessage(Commande.POST, CommandeType.REQUETE, pseudo + '\0' + txtMsg.Text);
            byte[] buffer = m.GetBytes();
            // Envoi
            clientSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, serverEP);
            txtMsg.Text = "";
        }

        /// <summary>
        /// Se declenche au click d'un bouton: Envoi du message GET/SUB/UNSUB....
        /// </summary>
        private void btnClickSENDMSG(object sender,EventArgs e)
        {
            // On recupere le tag
            int cmd = Convert.ToInt32(((Button)sender).Tag);
            // On crée le message
            ChatMessage m = new ChatMessage((Commande)cmd, CommandeType.REQUETE, txtMsg.Text);
            byte[] buffer = m.GetBytes();
            // Envoi du message
            clientSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, serverEP);

            // On affiche le nom de la room dans le titre du form
            if ((Commande)cmd == Commande.CREATEROOM)
                this.Text = "Room : " + txtMsg.Text;
        }

        /// <summary>
        /// On capture la touche entrée pour envoi le message POST
        /// </summary>
        private void txtMsg_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                btnPOST_Click(sender, e);
        }

        /// <summary>
        ///  On bloque la taille du message
        /// </summary>
        private void txtMsg_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (((TextBox)sender).Text.Length > 1400 && (e.KeyChar != (char)Keys.Return || e.KeyChar != (char)Keys.Enter))
                e.Handled = true;
        }
    }
}
