﻿using System;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace ServeurUdp
{
    class ServeurUdp
    {
        // Booleen utilisé dans la boucle while de readNet
        static bool listen = true;
        
        /* ROOMS */
        // Lien entre EndPoint et nom de room
        static Dictionary<EndPoint, string> roomAssign = new Dictionary<EndPoint, string>();
        // Lien entre nom de room et liste de endpoint dans cette room
        static Dictionary<string, List<EndPoint>> roomSub = new Dictionary<string, List<EndPoint>>();
        // Lien entre nom de room et liste de message de cette room
        static Dictionary<string, List<ChatMessage>> roomMessages = new Dictionary<string, List<ChatMessage>>();

        /// <summary>
        /// Renvoie la room du EndPoint
        /// </summary>
        /// <param name="ep">Endpoint de l'utilisateur</param>
        /// <returns>nom de la room</returns>
        static private string getRoom(EndPoint ep)
        {
            string room = "";
            if (roomAssign.ContainsKey(ep)) // Si l'utilisateur est dans une room, on envoie la room
                room = roomAssign[ep];
            else
                room = "global"; // sinon room global

            return room;
        }

        /// <summary>
        /// Envoie une reponse à un message
        /// </summary>
        /// <param name="servSocket">Socket du serveur</param>
        /// <param name="ep">Endpoint de l'utilisateur visé</param>
        /// <param name="cmd">Commande reçu</param>
        static void SendAnswer(Socket servSocket, EndPoint ep,Commande cmd)
        {
            byte[] buffer = ChatMessage.GetBytes(cmd, CommandeType.REPONSE, cmd.ToString() + '\0' + "OK");
            // On envoi l'accusé de reception
            servSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, ep);
        }
        /// <summary>
        /// Envoie tout les messages de la room du user
        /// </summary>
        /// <param name="servSocket">Socket du serveur</param>
        /// <param name="dest">Destinataire de la commande GET</param>
        static void SendAllMsg(Socket servSocket, EndPoint dest)
        {
            string room = getRoom(dest);

            foreach (ChatMessage cm in roomMessages[room])
            {
                // On convertie en byte chaque message
                byte[] buffer = cm.GetBytes();
                // Envoie du message
                int nBytes = servSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, dest);

            }
            Console.WriteLine("All messages send to " + dest);
        }
        /// <summary>
        /// Envoie du message au subscribers
        /// </summary>
        /// <param name="servSocket">Socket du serveur</param>
        /// <param name="dests">liste des sub</param>
        /// <param name="msg">Message a envoyer</param>
        static void SendToSub(Socket servSocket, List<EndPoint> dests, ChatMessage msg)
        {
            byte[] buffer;
            foreach (EndPoint ep in dests)
            {
                // Encodage du string dans un buffer de bytes en ASCII
                buffer = msg.GetBytes();
                // Envoie du message
                int nBytes = servSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, ep);
            }
            Console.WriteLine("Message send to subscribers");
        }
        /// <summary>
        /// Envoie un message à une personne
        /// </summary>
        /// <param name="servSocket">Socket du serveur</param>
        /// <param name="dests">destinataire</param>
        /// <param name="msg">Message a envoyer</param>
        static void SendToOne(Socket servSocket, EndPoint dest, ChatMessage msg)
        {
            byte[] buffer;
            buffer = msg.GetBytes();
            servSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, dest);
        }

        /// <summary>
        /// Sert a rejoindre une room / la crée
        /// </summary>
        /// <param name="roomName">Nom de la room</param>
        /// <param name="EP">Utilisateur</param>
        static void JoinRoom(string roomName,EndPoint EP)
        {
            foreach (KeyValuePair<string, List<EndPoint>> kv in roomSub) // on parcours les rooms
            {
                if (kv.Value.Contains(EP)) // Si l'utilisateur est deja dans une room
                    kv.Value.Remove(EP); // On le sort
            }

            if (!roomAssign.ContainsKey(EP)) // on met l'utilisateur dans une room
                roomAssign.Add(EP, roomName);
            else
                roomAssign[EP] = roomName;
            if (!roomMessages.ContainsKey(roomName)) // Si nouvelle room, on crée une liste de messages
                roomMessages.Add(roomName, new List<ChatMessage>());
            if (!roomSub.ContainsKey(roomName)) // Si nouvelle room, on crée une liste de sub
                roomSub.Add(roomName, new List<EndPoint>());
        }

        /// <summary>
        /// Ecoute le reseau et traite les message reçu
        /// </summary>
        /// <param name="serverSocket">Socket du serveur</param>
        static void readNet(Socket serverSocket)
        {
            byte[] buffer = new byte[1500];
            while (listen)
            {
                Console.WriteLine('\n' + "Attente d'une nouveau message...");

                // Reception message client et sauvegarde du dernier client
                EndPoint lastUser = new IPEndPoint(IPAddress.Any, 0);
                int nBytes = serverSocket.ReceiveFrom(buffer, buffer.Length, SocketFlags.None, ref lastUser);

                ChatMessage receive = new ChatMessage(buffer);

                // On recupere le contenu du message
                String msg = receive.data;

                // ========= Gestion de commande ===============
                switch (receive.commande)
                {
                    case Commande.POST:// Si post : On enregistre le message, on envoi au sub et on repond
                        string room = getRoom(lastUser);
                        Console.WriteLine(" <" + lastUser + "> " + msg);
                        roomMessages[room].Add(receive);
                        SendAnswer(serverSocket, lastUser, receive.commande);
                        SendToSub(serverSocket, roomSub[room], receive);
                        break;
                    case Commande.GET:// Si get : On envoie une reponse, et tout les messages
                        Console.WriteLine(" <" + lastUser + "> GET");
                        SendAnswer(serverSocket, lastUser, receive.commande);
                        SendAllMsg(serverSocket, lastUser);
                        break;
                    case Commande.SUBSCRIBE: // Si Sub : On rengistre le sub et on envoie la reponse de confirm
                        Console.WriteLine(" <" + lastUser + "> SUBSCRIBE");
                        SendAnswer(serverSocket, lastUser, receive.commande);
                        roomSub[getRoom(lastUser)].Add(lastUser);
                        break;
                    case Commande.UNSUBSCRIBE:// Si Sub : On rengistre le UNSUB et on envoie la reponse de confirm
                        Console.WriteLine(" <" + lastUser + "> UNSUBSCRIBE");
                        SendAnswer(serverSocket, lastUser, receive.commande);
                        roomSub[getRoom(lastUser)].Remove(lastUser);
                        break;
                    case Commande.STOPSERVEUR: // Si stopserveur : On stop le serv
                        Console.WriteLine(" <" + lastUser + "> STOPSERVEUR");
                        SendAnswer(serverSocket, lastUser, receive.commande);
                        stopServeur(ref serverSocket);                 
                        break;
                    case Commande.HELP: // Si help: on envoie les commandes possibles
                        Console.WriteLine(" <" + lastUser + "> HELP");
                        string help = '\n' + "0:POST" + '\n' + "1:GET" + '\n' + "2:HELP" + '\n' + "3:QUIT" + '\n' + "4:STOPSERVEUR" + '\n' + "5:SUBSCRIBE" + '\n' + "6:SUBSCRIBEv2" + '\n' + "7:UNSUBSCRIBE";
                        ChatMessage c = new ChatMessage(Commande.HELP, CommandeType.REPONSE, help);
                        SendToOne(serverSocket, lastUser, c);
                        break;
                    case Commande.CREATEROOM: // Si createroom : on fait rejoindre la room
                        Console.WriteLine(" <" + lastUser + "> CREATEROOM");
                        SendAnswer(serverSocket, lastUser, receive.commande);
                        JoinRoom(receive.data,lastUser);
                        break;

                }
                // ==========================================
            }
        }

        /// <summary>
        ///  Coupe le serveur et la socket
        /// </summary>
        /// <param name="socket">Socket du serveur</param>
        static void stopServeur(ref Socket socket)
        {
            Console.WriteLine("Fermeture Socket...");
            socket.Close();
            Console.WriteLine("============== SERVEUR OFF ==============" + '\n' + "Press enter to close this window...");

            Console.ReadKey(true);
            Environment.Exit(0);
        }

        /// <summary>
        ///  Programme principal
        /// </summary>
        static void Main(string[] args)
        {
            try
            {
                // On crée la room global
                if (!roomMessages.ContainsKey("global"))
                    roomMessages.Add("global", new List<ChatMessage>());
                if (!roomSub.ContainsKey("global"))
                    roomSub.Add("global", new List<EndPoint>());

                // ************************************************************** Initialisation

                Console.WriteLine("=============================================");
                Console.WriteLine("=============================================");
                Console.WriteLine("================= SERVEUR ===================");
                Console.WriteLine("=============================================");
                Console.WriteLine("=============================================" + '\n' + '\n');

                //// Création de la socket du serveur
                Socket serverSocket = new Socket(
                    AddressFamily.InterNetwork,
                    SocketType.Dgram,
                    ProtocolType.Udp);


                // Liaison de la socket au point de communication
                serverSocket.Bind(new IPEndPoint(IPAddress.Any, 11111));

                //************************************************************** Communications
                // Lancement du thread d'ecoute
                Thread listen = new Thread(x => readNet(serverSocket));
                listen.Start();
            }

            catch (SocketException E)
            {
                Console.WriteLine(E.Message);
                Console.ReadKey();
            }

            Console.ReadKey();
        }

    }
}
