﻿using System;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace ServeurUdp
{
    class ServeurUdp
    {

        static private List<ChatMessage> msgList;
        static private List<EndPoint> subList;
        static private Socket serverSocket;
        static private Dictionary<String, int> rooms;

        static void Room(int rootPort,int portRoom)
        {
            try
            {
                //// Création de la socket du serveur
                serverSocket = new Socket(
                    AddressFamily.InterNetwork,
                    SocketType.Dgram,
                    ProtocolType.Udp);
                msgList = new List<ChatMessage>();
                subList = new List<EndPoint>();

                if (rootPort == portRoom)
                    rooms = new Dictionary<string, int>();

                EndPoint servPrinc = new IPEndPoint(IPAddress.Parse("127.0.0.1"), rootPort);
                // Liaison de la socket au point de communication
                serverSocket.Bind(new IPEndPoint(IPAddress.Any, portRoom));

                byte[] buffer = new byte[1500];
                while (true)
                {
                    Console.WriteLine('\n' + "Attente d'une nouveau message...");

                    // Reception message client et sauvegarde du dernier client
                    EndPoint lastUser = new IPEndPoint(IPAddress.Any, 0);
                    int nBytes = serverSocket.ReceiveFrom(buffer, buffer.Length, SocketFlags.None, ref lastUser);

                    ChatMessage receive = new ChatMessage(buffer);

                    // On recupere le contenu du message
                    String msg = receive.data;

                    // ========= Gestion de commande ===============
                    switch (receive.commande)
                    {
                        case Commande.POST:// Si post : On enregistre le message, on envoi au sub et on repond
                            Console.WriteLine(" <" + lastUser + "> " + msg);
                            msgList.Add(receive);
                            SendAnswer(serverSocket, lastUser, receive.commande);
                            SendToSub(serverSocket, subList, receive);
                            break;
                        case Commande.GET:// Si get : On envoie une reponse, et tout les messages
                            Console.WriteLine(" <" + lastUser + "> GET");
                            SendAnswer(serverSocket, lastUser, receive.commande);
                            SendAllMsg(serverSocket, lastUser,msgList);
                            break;
                        case Commande.SUBSCRIBE: // Si Sub : On rengistre le sub et on envoie la reponse de confirm
                            Console.WriteLine(" <" + lastUser + "> SUBSCRIBE");
                            SendAnswer(serverSocket, lastUser, receive.commande);
                            subList.Add(lastUser);
                            break;
                        case Commande.UNSUBSCRIBE:// Si Sub : On rengistre le UNSUB et on envoie la reponse de confirm
                            Console.WriteLine(" <" + lastUser + "> UNSUBSCRIBE");
                            SendAnswer(serverSocket, lastUser, receive.commande);
                            subList.Remove(lastUser);
                            break;
                        case Commande.STOPSERVEUR: // Si stopserveur : On stop le serv
                            Console.WriteLine(" <" + lastUser + "> STOPSERVEUR");
                            SendAnswer(serverSocket, lastUser, receive.commande);
                            stopServeur(ref serverSocket);
                            break;
                        case Commande.HELP: // Si help: on envoie les commandes possibles
                            Console.WriteLine(" <" + lastUser + "> HELP");
                            string help = '\n' + "0:POST" + '\n' + "1:GET" + '\n' + "2:HELP" + '\n' + "3:QUIT" + '\n' + "4:STOPSERVEUR" + '\n' + "5:SUBSCRIBE" + '\n' + "6:SUBSCRIBEv2" + '\n' + "7:UNSUBSCRIBE";
                            ChatMessage c = new ChatMessage(Commande.HELP, CommandeType.REPONSE, help);
                            SendToOne(serverSocket, lastUser, c);
                            break;
                        case Commande.CREATEROOM: // Si createroom : on fait rejoindre la room
                            Console.WriteLine(" <" + lastUser + "> CREATEROOM");
                            if(rootPort == portRoom)
                            {
                                if(rooms.ContainsKey(msg))
                                    SendToOne(serverSocket, lastUser, new ChatMessage(Commande.CREATEROOM, CommandeType.REPONSE, rooms[msg].ToString()));
                                else
                                {
                                    int nextPort = 0;
                                    // TESTER SI LASTPORT+1 est libre
                                }
                            }
                            // ???
                            break;

                    }
                    // ==========================================
                }
            }
            catch(Exception e)
            {
                Console.WriteLine(e.Message);
            }
        }

        /// <summary>
        /// Envoie une reponse à un message
        /// </summary>
        /// <param name="servSocket">Socket du serveur</param>
        /// <param name="ep">Endpoint de l'utilisateur visé</param>
        /// <param name="cmd">Commande reçu</param>
        static void SendAnswer(Socket servSocket, EndPoint ep,Commande cmd)
        {
            byte[] buffer = ChatMessage.GetBytes(cmd, CommandeType.REPONSE, cmd.ToString() + '\0' + "OK");
            // On envoi l'accusé de reception
            servSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, ep);
        }
        /// <summary>
        /// Envoie tout les messages de la room du user
        /// </summary>
        /// <param name="servSocket">Socket du serveur</param>
        /// <param name="dest">Destinataire de la commande GET</param>
        static void SendAllMsg(Socket servSocket, EndPoint dest, List<ChatMessage> msgs)
        {
            foreach (ChatMessage cm in msgs)
            {
                // On convertie en byte chaque message
                byte[] buffer = cm.GetBytes();
                // Envoie du message
                int nBytes = servSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, dest);

            }
            Console.WriteLine("All messages send to " + dest);
        }
        /// <summary>
        /// Envoie du message au subscribers
        /// </summary>
        /// <param name="servSocket">Socket du serveur</param>
        /// <param name="dests">liste des sub</param>
        /// <param name="msg">Message a envoyer</param>
        static void SendToSub(Socket servSocket, List<EndPoint> dests, ChatMessage msg)
        {
            byte[] buffer;
            foreach (EndPoint ep in dests)
            {
                // Encodage du string dans un buffer de bytes en ASCII
                buffer = msg.GetBytes();
                // Envoie du message
                int nBytes = servSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, ep);
            }
            Console.WriteLine("Message send to subscribers");
        }
        /// <summary>
        /// Envoie un message à une personne
        /// </summary>
        /// <param name="servSocket">Socket du serveur</param>
        /// <param name="dests">destinataire</param>
        /// <param name="msg">Message a envoyer</param>
        static void SendToOne(Socket servSocket, EndPoint dest, ChatMessage msg)
        {
            byte[] buffer;
            buffer = msg.GetBytes();
            servSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, dest);
        }
        /// <summary>
        ///  Coupe le serveur et la socket
        /// </summary>
        /// <param name="socket">Socket du serveur</param>
        static void stopServeur(ref Socket socket)
        {
            Console.WriteLine("Fermeture Socket...");
            socket.Close();
            Console.WriteLine("============== SERVEUR OFF ==============" + '\n' + "Press enter to close this window...");

            Console.ReadKey(true);
            Environment.Exit(0);
        }
        static void createRoom()
        {

        }


        /// <summary>
        ///  Programme principal
        /// </summary>
        static void Main(string[] args)
        {
            try
            {
                // ************************************************************** Initialisation

                Console.WriteLine("=============================================");
                Console.WriteLine("=============================================");
                Console.WriteLine("================= SERVEUR ===================");
                Console.WriteLine("=============================================");
                Console.WriteLine("=============================================" + '\n' + '\n');

                Room(11111,11111);
                

            }

            catch (SocketException E)
            {
                Console.WriteLine(E.Message);
                Console.ReadKey();
            }
            Console.ReadKey();
        }

    }
}
