﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TronClient
{
    public enum Commande
    {
        POST, GET, HELP, QUIT, STOPSERVEUR, SUBSCRIBE, SUBSCRIBEv2, UNSUBSCRIBE, CREATEROOM
    };

    public enum CommandeType { REQUETE, REPONSE };

    class ChatMessage
    {
        public const int bufferSize = 1500;

        public Commande commande;               // commande
        public CommandeType commandeType;       // type (Requête/Réponse)
        public int dataSize;                    // taille de la donnée
        public String data;                     // données de la commande
        public String message;                  // Message
        public String pseudo;                   // Pseudo

        public ChatMessage(Commande commande, CommandeType type, String data)
        {
            this.commande = commande;
            this.commandeType = type;
            this.dataSize = data.Length;
            this.data = data;
            if (data.Contains('\0'))
            {
                this.pseudo = data.Split('\0')[0];
                this.message = data.Split('\0')[1];
            }
            else
            {
                this.pseudo = null;
                this.message = data;
            }
        }

        public ChatMessage(byte[] buffer)
        {
            this.commande = (Commande)buffer[0];
            this.commandeType = (CommandeType)buffer[1];
            this.dataSize = BitConverter.ToInt32(buffer, 2);
            this.data = Encoding.ASCII.GetString(buffer, sizeof(Int32) + 2, dataSize);
            if (data.Contains('\0'))
            {
                this.pseudo = data.Split('\0')[0];
                this.message = data.Split('\0')[1];
            }
            else
            {
                this.pseudo = null;
                this.message = data;
            }
        }

        public byte[] GetBytes()
        {
            byte[] buf = new byte[bufferSize];

            buf[0] = Convert.ToByte((char)this.commande); // Commande
            buf[1] = Convert.ToByte((char)this.commandeType); // Type de commande

            int i = 2;
            foreach (Byte b in BitConverter.GetBytes(dataSize)) // Taille des données
            {
                buf[i] = b;
                i++;
            }
            foreach (Byte b in System.Text.Encoding.ASCII.GetBytes(data)) // Données
            {
                buf[i] = b;
                i++;
            }

            return buf;
        }

        public static byte[] GetBytes(Commande commande, CommandeType type, String data)
        {
            ChatMessage chatCommande = new ChatMessage(commande, type, data);
            return chatCommande.GetBytes();
        }

        public override string ToString()
        {
            return "[" + commande + "," + commandeType + ",\"" + pseudo + "\"," + dataSize + ",\"" + message + "\"]";
        }

    }
}
