﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Windows.Forms;

namespace TronClient
{
    public class Client
    {
        private Tron.Tron myTron;        // Moteur du jeu

        public byte frequence;      // Temps du tour de jeu (en dixieme de s)
        Socket clientSocket;
        string serverIP;
        int serverPort;

        // constructeur : IP/Port du serveur
        public Client(String myServerIP, int myServerPort)
        {
            // Creation de la socket d'écoute TCP
            clientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            serverIP = myServerIP;
            serverPort = myServerPort;
        }


        // Appelé au début de la partie
        public Tron.Tron Init()
        {
            try
            {
                System.Console.WriteLine("Init");
                byte[] buffer = new byte[4];

                // Connexion au serveur   
                clientSocket.Bind(new IPEndPoint(IPAddress.Any, 0));
                IPEndPoint serv = new IPEndPoint(IPAddress.Parse(serverIP), serverPort);
                clientSocket.Connect(serv);

                // Réception des paramètres
                int nBytes = clientSocket.Receive(buffer, buffer.Length, SocketFlags.None);

                // Initialisation de la fréquence : frequence = <frequence>
                frequence = buffer[3];

                // Initialisation du moteur : myTron = new Tron(byte <taille terrain>, byte <nombre de joueurs>, byte <numéro du joueur>);
                // buffer[1] = taille
                // buffer[0] = nbJoueur
                // buffer[2] = NumJoueur
                myTron = new Tron.Tron(buffer[1], buffer[0], buffer[2]);
            }
            catch(System.Net.Sockets.SocketException E)
            {
                MessageBox.Show("Adresse du serveur incorrecte");
                clientSocket.Close();
                Environment.Exit(1);
            }

            // Retourne le moteur
            return myTron;

        }

        // Appelé régulièrement à chaque tour de jeu
        public void Routine()
        {
            System.Console.WriteLine("Routine");

            byte[] bufferSend = new byte[2];
            byte[] bufferReceive = new byte[Convert.ToInt32(myTron.getNJoueurs())];

            // Envoie de sa direction : myTron.getDirection()
            bufferSend[0] = myTron.getMonNum();
            bufferSend[1] = myTron.getDirection();

            clientSocket.Send(bufferSend, 0, bufferSend.Length, SocketFlags.None);

            // Reception de toutes les directions : myTron.setDirections(byte[] < toutes les directions>);
            int nBytes = clientSocket.Receive(bufferReceive, bufferReceive.Length, SocketFlags.None);
            myTron.setDirections(bufferReceive);
        }

        // Appelé à la fin de la partie
        public void Conclusion()
        {
            System.Console.WriteLine("Conclusion");
            clientSocket.Close();
            // fermeture socket
        }


        // propriété frequence (Ne pas toucher)
        public int freq
        {
            get { return frequence * 100; }
            set { }
        }
    }
}
