﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace TronClient
{
    // Form du lobby
    public partial class FormLobby : Form
    {
        Client myClient;

        public FormLobby()
        {
            InitializeComponent();

            chat();
        }

        // Lance la partie
        private void StartTron()
        {
            // Création du client avec IP et Port
            myClient = new Client(textBoxIP.Text, int.Parse(textBoxPort.Text));

            // Création et affichage de la Form de jeu
            FormTron myFormTron = new FormTron(myClient);
            myFormTron.Show();
        }

        // Appelé au click sur le bouton start
        private void button1_Click(object sender, EventArgs e)
        {
            // Lance la partie
            StartTron();
        }

        // ========================== CHAT ==========================

        Socket clientSocket;
        EndPoint serverEP;

        private void readNet(Socket client, EndPoint server)
        {
            // Gets executed on a seperate thread and 
            // doesn't block the UI while sleeping

            try
            {

                byte[] buffer = new byte[1500];
                while (true)
                {
                    int nBytes = client.ReceiveFrom(buffer, buffer.Length, SocketFlags.None, ref server);
                    ChatMessage receive = new ChatMessage(buffer);
                    Action action;

                    // On vide la rtb si on recoit la reponse d'un GET
                    if (receive.commandeType == CommandeType.REPONSE && receive.commande == Commande.GET)
                    {
                        action = () => rtbMSG.Text = "";
                        this.Invoke(action);
                    }

                    if (receive.commandeType == CommandeType.REPONSE)
                    {
                        Console.WriteLine(" < Reponse du serveur > " + receive.data);
                    }
                    else
                    {
                        string[] splitted = receive.data.Split('\0');

                        if (receive.commande == Commande.GET)
                        {
                            action = () => rtbMSG.Text = '\n' + " <" + splitted[0] + "> " + splitted[1];
                            this.Invoke(action);
                        }
                        else
                        {
                            action = () => rtbMSG.Text += '\n' + " <" + splitted[0] + "> " + splitted[1];
                            this.Invoke(action);
                            Console.WriteLine(" <" + splitted[0] + "> " + splitted[1]);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("ERROR READ NET : Serveur fermé");
                MessageBox.Show("Le serveur n'est pas disponible! Fermeture du client...");
                Application.Exit();
            }

        }

        private void btnPOST_Click(object sender, EventArgs e)
        {
            ChatMessage m = new ChatMessage(Commande.POST, CommandeType.REQUETE, txtPseudo.Text + '\0' + txtMsg.Text);
            byte[] buffer = m.GetBytes();
            Console.WriteLine("taille du buffer" + buffer.Length);
            clientSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, serverEP);
            txtMsg.Text = "";
        }

        private void btnClickSENDMSG(object sender, EventArgs e)
        {
            int cmd = Convert.ToInt32(((Button)sender).Tag);
            ChatMessage m = new ChatMessage((Commande)cmd, CommandeType.REQUETE, txtMsg.Text);
            byte[] buffer = m.GetBytes();
            Console.WriteLine("taille du buffer" + buffer.Length);
            clientSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, serverEP);

            if ((Commande)cmd == Commande.CREATEROOM)
                this.Text = "Room : " + txtMsg.Text;
        }

        private void txtMsg_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                btnPOST_Click(sender, e);
        }

        private void txtMsg_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (((TextBox)sender).Text.Length > 1400 && (e.KeyChar != (char)Keys.Return || e.KeyChar != (char)Keys.Enter))
                e.Handled = true;
        }

        private void chat()
        {

            Console.WriteLine("=============================================");
            Console.WriteLine("=============================================");
            Console.WriteLine("================== CLIENT ===================");
            Console.WriteLine("=============================================");
            Console.WriteLine("=============================================" + '\n' + '\n');

            string serverIP = "10.0.0.1";
            int serverPort = 8081;

            // Création de la socket d'écoute UDP
            clientSocket = new Socket(
                AddressFamily.InterNetwork,
                SocketType.Dgram,
                ProtocolType.Udp);

            // Liaison de la socket au point de communication
            clientSocket.Bind(new IPEndPoint(IPAddress.Any, 0));

            // Création du EndPoint serveur
            serverEP = new IPEndPoint(IPAddress.Parse(serverIP), serverPort);

            Thread sendAll = new Thread(x => readNet(clientSocket, serverEP));
            sendAll.Start();

            this.ActiveControl = txtMsg;
        }
    }
}
